<?php
namespace Fiscal\Entities;

use \Geral\Util\Ambiente;


/**
 * @method \Fiscal\Repositories\Notafiscal getRepository static
 * @method \Fiscal\Entities\Notafiscal findOneByCdfilialAndNrlanctonfAndNrorg($cdfilial, $nrlanctonf, $nrorg) static
 * @method array findByIddocreplicAndNrorg($iddocreplic, $nrorg) static
 * @method \Fiscal\Entities\Notafiscal getObjectByWhere static
 */
class Notafiscal extends Base\Notafiscal
{
    /* @var $contopectbl \Contabilidade\Entities\Contopectbl */
    protected  $contopectbl;
    /**@var $contasReceber array[] \Financeiro\Entitites\ContaReceber*/
    private $contasReceber;
    /**@var $validanftpop \Indeterminado\Entities\Validanftpop*/
    private $validanftpop;
    /**@var $lanctoestoq \Estoque\Entities\Lanctoestoq*/
    private $lanctoestoq;
    /**
     * Prefixo do CDCONTADOR de notafiscal. Para completar o contador usado, este deve ser incrementado com a filial
     * da unidade corrente.
     */
    const CDCONTADOR_NOTAFISCAL   = "LANCNF/FILIAL";

    /**
     * Prefixo do CDCONTADOR de lancamento de estoque. Para completar o contador usado, este deve ser incrementado com
     * a filial da unidade corrente.
     */
    const CDCONTADOR_LANC_ESTOQUE = "LANCTOEST";

    /**
     * Identificador de nota fiscal de entrada.
     */
    const NOTA_FISCAL_ENTRADA = "E";
    /**
     * Identificador de nota fiscal de saida.
     */
    const NOTA_FISCAL_SAIDA   = "S";

    /**
     * IDBLOQUEADO = S: Indica que a nota fiscal esta bloqueada.
     */
    const NOTA_FISCAL_BLOQUEADA = "S";
    /**
     * IDBLOQUEADO = N: Indica que a nota fiscal nao esta bloqueada.
     */
    const NOTA_FISCAL_NAO_BLOQUEADA = "N";

    /**
     * IDDOCFISCAL = S: Identifica a nota como um documento fiscal.
     */
    const DOCUMENTO_FISCAL     = "S";
    /**
     * IDDOCFISCAL = S: Identifica a nota como nao sendo um documento fiscal.
     */
    const DOCUMENTO_FISCAL_NAO = "N";

    /**
     * IDDOCFINANC = S: Identifica a nota como documento financeiro.
     */
    const DOCUMENTO_FINANCEIRO     = "S";
    /**
     * IDDOCFINANC = N: Identifica a nota como nao financeiro.
     */
    const DOCUMENTO_NAO_FINANCEIRO = "N";

    /**
     * Status da nota fiscal: Pendente.
     */
    const PENDENTE     = "P";
    /**
     * Status da nota fiscal: Aprovado.
     */
    const APROVADO     = "A";
    /**
     * Status da nota fiscal: Emitido.
     */
    const EMITIDO      = "E";
    /**
     * Status da nota fiscal: Cancelado.
     */
    const CANCELADO    = "C";
    /**
     * Status da nota fiscal: Previsto.
     */
    const PREVISTO     = "V";
    /**
     * Status da nota fiscal: Gerencial.
     */
    const GERENCIAL    = "G";
    /**
     * Status da nota fiscal: Rateio.
     */
    const RATEIO       = "R";
    /**
     * Status da nota fiscal: Liberado.
     */
    const LIBERADO     = "L";
    /**
     * Status da nota fiscal: em digitacao
     */
    const EM_DIGITACAO = "D";

    /**
     * IDSISTEMA = DFE: Identifica o sistema ao qual esta nota pertence. Nesta caso, pertence ao DFE (documentos fiscais de entrada.
     */
    const PREFIXO_SISTEMA_DFE = "DFE";

    /**
     * IDIMPRESSODANFE = S: Identifica um documento como impresso.
     */
    const DOCUMENTO_IMPRESSO = "S";
    /**
     * IDIMPRESSODANFE = N: Identifica um documento como nao impresso.
     */
    const DOCUMENTO_NAO_IMPRESSO = "N";

    /**
     * IDEXPNFEP = S: Identificador de exportacao Paulistana.
     */
    const NFE_PAULISTANA     = "S";
    /**
     * IDEXPNFEP = N: Identificador de exportacao Paulistana.
     */
    const NFE_PAULISTANA_NAO = "N";

    /**
     * Idenficador de exportacao da NF-e Santo Andre.
     */
    const IDEXPNFESANAND_SIM = "S";
    /**
     * Idenficador de exportacao da NF-e Santo Andre.
     */
    const IDEXPNFESANAND_NAO = "N";

    /**
     * IDEXPNFEMANAUS = S: Identificador de exportacao Paulistana.
     */
    const NFE_MANAUS     = "S";
    /**
     * IDEXPNFEMANAUS = N: Identificador de exportacao Paulistana.
     */
    const NFE_MANAUS_NAO = "N";

    /**
     * IDSTATUSCTBNF: Identificador de nota fiscal de contabilizacao pendente.
     */
    const STATUS_CONTABILIZACAO_PENDENTE       = "P";
    /**
     * IDSTATUSCTBNF: Identificador de nota fiscal de contabilizacao contabilizado..
     */
    const STATUS_CONTABILIZACAO_CONTABILIZADO  = "C";
    /**
     * IDSTATUSCTBNF: Identificador de nota fiscal de contabilizacao erro.
     */
    const STATUS_CONTABILIZACAO_ERRO           = "E";
    /**
     * IDSTATUSCTBNF: Identificador de nota fiscal de contabilizacao recontabilizar..
     */
    const STATUS_CONTABILIZACAO_RECONTABILIZAR = "P";

    /**
     * IDSTATUS = P: NF pendente
     */
    const STATUS_PENDENTE = "P";
    /**
     * IDSTATUS = A: NF aprovada
     */
    const STATUS_APROVADO = "A";
    /**
     * IDSTATUS = E: NF emitida
     */
    const STATUS_EMITIDO = "E";
    /**
     * IDSTATUS = C: NF cancelada
     */
    const STATUS_CANCELADO = "C";
    /**
     * IDSTATUS = V: NF previsto.
     */
    const STATUS_PREVISTO = "V";
    /**
     * IDSTATUS = G: NF gerencial.
     */
    const STATUS_GERENCIAL = "G";
    /**
     * IDSTATUS = R: NF rateio.
     */
    const STATUS_RATEIO = "R";
    /**
     * IDSTATUS = L: NF liberado.
     */
    const STATUS_LIBERADO = "L";

    /**
     * IDEMAILNFETRANSP: Identificador de envio de email para a transportadora pendente.
     */
    const ENVIO_EMAIL_TRANSPORTADORA_PENDENTE  = "P";
    /**
     * IDEMAILNFETRANSP: Identificador de envio de email para a transportadora executado.
     */
    const ENVIO_EMAIL_TRANSPORTADORA_EXECUTADO = "E";
    /**
     * IDEMAILNFETRANSP: Identificador de envio de email para a transportadora nao se aplica.
     */
    const ENVIO_EMAIL_NAO_SE_APLICA            = "N";

    /**
     * IDEMAILNFE: Identificador de envio de email para a transportadora nao se aplica.
     */
    const ENVIO_EMAIL_EXECUTADO            = "E";

    /**
     * IDEMAILNFE: Identificador de envio de email para a transportadora nao se aplica.
     */
    const ENVIO_EMAIL_PENDENTE            = "P";

    /**
     * IDVERIFNFCONF = S: Verificar se a NF foi devidamente conferida (valores, impostos cadastrados, cálculos e outros).
     */
    const NOTA_CONFERIDA = "S";
    /**
     * IDVERIFNFCONF = N: Verificar se a NF foi nao devidamente conferida (valores, impostos cadastrados, cálculos e outros).
     */
    const NOTA_NAO_CONFERIDA = "N";

    /**
     * IDNFDISPFIN = S: Disponivel para o financeiro.
     */
    const DISPONIVEL_PARA_FINANCEIRO     = "S";
    /**
     * IDNFDISPFIN = N: Nao disponivel para o financeiro.
     */
    const NAO_DISPONIVEL_PARA_FINANCEIRO = "N";

    /**
     * Identificador de ambiente 0.
     */
    const IDTPAMBNFE_0 = "0";
    /**
     * Identificado de ambiente 1.
     */
    const IDTPAMBNFE_1 = "1";
    /**
     * Identificador de ambiente 2.
     */
    const IDTPAMBNFE_2 = "2";

    /**
     * IDBLOQNFFIN = B:Indifica o bloqueio da NF no financeiro
     */
    const NF_BLOQUEADA_NO_FINANCEIRO = "B";
    /**
     * IDBLOQNFFIN = L:Indifica o bloqueio da NF no financeiro
     */
    const NF_LIBERADA_NO_FINANCEIRO  = "L";

    /**
     * IDNFGERADAVIAXML = S: de Nota Fiscal Gerada via XML
     */
    const NF_GERADA_VIA_XML = "S";
    /**
     * IDNFGERADAVIAXML = N: de Nota Fiscal Gerada via XML
     */
    const NF_NAO_GERADA_VIA_XML = "N";

    /**
     * IDNFCOMPRALEG = S: Identificador de nf de compra legal
     */
    const NF_COMPRA_LEGAL_SIM = "S";
    /**
     * IDNFCOMPRALEG = N: Identificador de nf de compra legal
     */
    const NF_COMPRA_LEGAL_NAO = "N";

    /**
     * IDDEBICREDNF = D: Identificador de debito na nota fiscal.
     */
    const ID_DEBITO_NOTA_FISCAL        = "D";
    /**
     * IDDEBICREDNF = C: Identificador de credito na nota fiscal.
     */
    const ID_CREDITO_NOTA_FISCAL       = "C";
    /**
     * IDDEBICREDNF = N: Nao há identificador na nota.
     */
    const ID_DEBITO_NENHUM_NOTA_FISCAL = "N";

    /**
     * IDSTATUSNFE = P: nfe pendente.
     */
    const ID_STATUS_NFE_PENDENTE     = "P";
    /**
     * IDSTATUSNFE = T: nfe transmitida.
     */
    const ID_STATUS_NFE_TRANSMITIDA  = "T";
    /**
     * IDSTATUSNFE = A: nfe aceita.
     */
    const ID_STATUS_NFE_ACEITA       = "A";
    /**
     *  IDSTATUSNFE = R: nfe recusada.
     */
    const ID_STATUS_NFE_RECUSADA     = "R";
    /**
     *  IDSTATUSNFE = D: nfe de uso denegado.
     */
    const ID_STATUS_NFE_USO_DENEGADO = "D";

    /**
     *   IDORIGDESTNF = L: Filial
     */
    const ID_ORIGEM_DESTINO_FILIAL      = "L";
    /**
     *   IDORIGDESTNF = F: Fornecedor
     */
    const ID_ORIGEM_DESTINO_FORNECEDOR  = "F";
    /**
     *   IDORIGDESTNF = C: Cliente
     */
    const ID_ORIGEM_DESTINO_CLIENTE     = "C";
    /**
     *   IDORIGDESTNF = U: Licenca de Uso
     */
    const ID_ORIGEM_DESTINO_LICENCA_USO = "U";

    const MODO_SERIE_NORMAL = 1;
    const MODO_SERIE_SCAN   = 3;
    const MODO_SERIE_EPEC   = 4;
    const MODO_SERIE_SVAN_AN   = 6;
    const MODO_SERIE_SVAN_RS   = 7;

    const TIPO_INSCRICAO_CPF   = "CPF";
    const TIPO_INSCRICAO_CNPJ  = "CNPJ";
    const TIPO_INSCRICAO_OUTRO = "OUTRO";

    const NFE_VERSAO_200 = "2.00";
    const NFE_VERSAO_310 = "3.10";

    /**
     * CDTIPOOPER = '01' Nota Fiscal de Venda
     */
    const NF_SAIDA_VENDA = '01';
    /**
     * CDTIPOOPER = '02' Nota Fiscal de Transferência
     */
    const NF_SAIDA_TRANSFERENCIA = '02';

    /**
     * Tipo de emissao da nota fiscal: Não EmitidaEMAI.
     */
    const IDTIPOEMISNFE_NAO_EMITIDO = '0';

    /**
     * Tipo de emissao da nota fiscal: normal.
     */
    const IDTIPOEMISNFE_NORMAL       = "1";
    /**
     * Tipo de Emissao da nota fiscal: contingencia.
     */
    const IDTIPOEMISNFE_CONTINGENCIA = "2";

    /**
     * Tipo de Emissao da nota fiscal: contingencia scan
     */
    const IDTIPOEMISNFE_CONTINGENCIA_SCAN = "3";



    /**
     * @param $data
     */
    public function atualizaDataNF($data){
        $this->setDtcalcrecbruta($data);
        $this->persist();

        /** @todo Comentário: Érika Souza 12/09/2013 12:52
         * Não encontrei nenhum outro lugar que está sendo usado além
         * da tela de Fechamento do Dia.
         */
        //        /** @var $notafiscal Notafiscal */
        //        $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($this->cdfilial, $this->nrlanctonf, $this->nrorg);
        //        $notafiscal->getEmpresa()->getInscmuni();
    }



    /**
     * @return \Indeterminado\Entities\Validanftpop
     */
    public function getValidanftpop()
    {
        if (!isset($this->validanftpop)) {
            $params =  array(
                "identrsaidop" => $this->identrsaidop,
                "cdtipooper" => $this->cdtipooper,
                "nrorg" => $this->nrorg,
            );

            $this->validanftpop = \Indeterminado\Entities\Validanftpop::getRepository()->findBy($params);
        }

        return $this->validanftpop;
    }

    /**
     * @return \Fiscal\Entities\Notafiscal
     */
    public function setVdTipoOperac(\Geral\Entities\Tipooperac $tipoOperac) {
        return $this->setEntrsaidop($tipoOperac);
    }

    /**
     * @return \Geral\Entities\Tipooperac
     */
    public function getTipoOperac()
    {
        return $this->getEntrsaidop();
    }

    /**
     * @return \Fiscal\Entities\Notafiscal
     */
    public function setTipoOperac(\Geral\Entities\Tipooperac $tipoOperac) {
        return $this->setEntrsaidop($tipoOperac);
    }

    /**
     * @return array[] \Financeiro\Entities\Contareceber
     */
    public function getContasReceber()
    {
        if (!isset($this->contasReceber)) {
            $params =  array(
                "cdfilial" => $this->cdfilial,
                "nrlanctonf" => $this->nrlanctonf,
                "nrorg" => $this->nrorg,
            );

            $this->contasReceber = \Financeiro\Entities\Contareceber::getRepository()->findBy($params);
        }

        return $this->contasReceber;
    }

    /**
     * @return float
     */
    public function somaContasReceber()
    {
        $total = 0.00;
        $arContasRec = $this->getContasReceber();
        /**@var $contaRec \Financeiro\Entities\Contareceber*/
        foreach ($arContasRec as $contaRec) {
            $total += $contaRec->getVrorigrec();
        }

        return $total;
    }

    /**
     * @return array
     */
    public function getImpostosObrigatorios()
    {
        return \Indeterminado\Entities\Impobrigtpoper::findByIdentrsaidopAndCdtipooperAndNrorg($this->identrsaidop, $this->cdtipooper, $this->nrorg);
    }

    public function getContopectbl(){
        if ($this->contopectbl === null) {
            $id = array(
                'nrorg' => $this->nrorg,
                'cdempresa' => $this->cdempresa
            );
            $this->contopectbl = \Contabilidade\Entities\Contopectbl::getObject($id);
        }

        return $this->contopectbl;
    }

    /**
     * @return \Estoque\Entities\Lanctoestoq
     */
    public function getLanctoestoq() {
        if ($this->lanctoestoq === null) {
            $this->lanctoestoq = \Estoque\Entities\Lanctoestoq::getObjectByWhere(
                'NRORG = ? AND NRLANCTONF = ? AND CDFILIAL = ?',
                array(
                    $this->nrorg, $this->nrlanctonf, $this->cdfilial,
                )
            );
        }

        return $this->lanctoestoq;
    }

    /**
     * @static
     * @param $nrorg
     * @param $cdempresa
     * @return int|string
     */
    public static function geraNovoCodigoLoteNFE($nrorg, $cdempresa) {
        return \Geral\Util\NovoCodigo::geraCodigo($nrorg, "LOTENFE/FILIAL".$cdempresa,  1, 15);
    }

    /**
     * @static
     * @param $nrorg
     * @param $cdempresa
     * @param $nrinscricao
     * @param $serie
     * @param $tipoAmbiente
     * @param $nrinscricao
     * @return int|string
     */
    public static function geraNovoCodigoNFE($nrorg, $cdempresa, $nrinscricao, $serie, $tipoAmbiente){
        $id = "NOTAFISCALELE$cdempresa$nrinscricao$serie$tipoAmbiente";
        return \Geral\Util\NovoCodigo::geraCodigo($nrorg, $id, 1, 9);
    }

    /**
     * Recebe o numero da nota fiscal sem a formatacao necessária. Retorna o mesmo numero formatado segundo os
     * parametros da filial para numero de notas fiscais.
     *
     * @todo Este metodo deve ser feito respeitando o atributo NRTAMNUMNFUT da classe paramfilial.
     *
     * @static
     * @param $nrnotafisc
     *
     * @return string
     */
    public static function retornaNumeroNotaFormatado($nrnotafisc)
    {
        return $nrnotafisc;
    }

    /**
     * Retorna um nrlanconf sequencial, gerado pela procedure novocodigo.
     *
     * @static
     * @return int|string
     */
    public static function geraNovoNumeroLancamento()
    {
        $cdcontadorNF = Notafiscal::CDCONTADOR_NOTAFISCAL . \Geral\Util\Ambiente::getCdFilial();
        return \Geral\Util\NovoCodigo::geraCodigo(\Geral\Util\Ambiente::getNrorg(), $cdcontadorNF);
    }

}