<?php

/**
 * GepFluxoencasoliLogic.
 *
 * @package Logic
 * @author  erikasouza
 */
class GepFluxoencasoliLogic{


    CONST DUPLICADO = 'ERROR';

    private $_GepFluxoencasoliEntity;


    /**
     * Construtor da classe.
     */
    public function __construct(){
        $this->_GepFluxoencasoliEntity = new GepFluxoencasoliEntity();
    }


    /**
     * Gera novo c?digo do fluxo de encaminhamento da solicita??o.
     *
     * @param integer $nrorg N?mero da Organiza??o.
     *
     * @return Ambigous <number, number>
     */
    public static function novoCodigoGepFluxoencasoli($nrorg){
        $codigo = GepFluxoencasoliEntity::novoCodigoGepFluxoencasoli($nrorg);
        return $codigo;
    }


    /**       ok
     * Insere registro do fluxo de encaminhamento de uma determinada solicita??o.
     *
     * @param integer $solicitacao    Numero da solicitacao.
     * @param integer $parceiro       Codigo do parceiro.
     * @param integer $perfil         Perfil de acesso responsivel.
     * @param integer $tiposoli_ant   Numero do tipo da solicitacao anterior.
     * @param integer $situacao_ant   Numero da situacao anterior.
     * @param integer $tiposoli_prox  Numero do proximo tipo da solicitacao.
     * @param integer $situacao_prox  Numero da poroxima situacao.
     * @param integer $nrmotivo_fluxo Numero do motivo.
     * @param string  $txt_observacao Texto de observacao do parceiro.
     * @param string  $txt_log        Texto de log.
     * @param integer $nrorg          Numero da Organizacao.
     * @param date    $data           Data de insercao.
     * @param integer $operador       Codigo do operador que inseriu.
     *
     * @return boolean
     */
    public function insereFluxoencasoli(
        $solicitacao, $parceiro, $perfil, $tiposoli_ant, $situacao_ant,
        $tiposoli_prox, $situacao_prox, $nrmotivo_fluxo, $txt_observacao,
        $txt_log, $nrorg, $data, $operador
    ){
        $codigo = $this->_GepFluxoencasoliEntity->novoCodigoGepFluxoencasoli($nrorg);
        $insere = $this->_GepFluxoencasoliEntity->insereFluxoencasoli(
            $codigo, $solicitacao, $parceiro, $perfil, $tiposoli_ant, $situacao_ant,
            $tiposoli_prox, $situacao_prox, $nrmotivo_fluxo, $txt_observacao,
            $txt_log, $nrorg, $data, $operador
        );
        return $insere;
    }


    /**
     * Atualiza o fluxo de encaminhamento de uma determinada solicita??o.
     *
     * @param integer $nrorg             N?mero da Organiza??o.
     * @param integer $nrencaminhamento  C?digo do encaminhamento a ser atualizado.
     * @param integer $solicitacao       N?mero da solicita??o.
     * @param date    $dtinclusao        Data de inclus?o.
     * @param integer $motivo            N?mero do motivo do encaminhamento.
     * @param string  $observacao        Observa??o do Operador.
     * @param date    $dtatualiza        Data de atualiza??o.
     * @param integer $operador_atualiza C?digo do operador que atualizou.
     *
     * @return boolean
     */
    public function atualizaFluxoEncaminhamento(
        $nrorg, $nrencaminhamento, $solicitacao, date $dtinclusao,
        $motivo, $observacao, date $dtatualiza, $operador_atualiza
    ){
        $insere = $this->_GepFluxoencasoliEntity->atualizaFluxoEncaminhamento(
            $nrorg, $nrencaminhamento, $solicitacao, $dtinclusao, $motivo, $observacao, $dtatualiza, $operador_atualiza
        );

        return $insere;
    }


    /**
     * Retorna dados de encaminhamento de uma determinada solicitacao.
     *
     * @param integer $nrorg       N?mero da Organiza??o.
     * @param integer $solicitacao N?mero da Solicita?a?.
     *
     * @return GepFluxoencasoliEntity
     */
    public function retornaDadosEncaminhamento($nrorg, $solicitacao){
        try {
            $dados = $this->_GepFluxoencasoliEntity->consultaDadosEncaminhamento($nrorg, $solicitacao);
            return $dados;
        } catch (Exception $e) {
            switch ($e->getCode()) {
            case GepFluxoencasoliEntity::SEM_REGISTRO:
                return GepFluxoencasoliEntity::SEM_REGISTRO;
            default:
                tk_message('Erro desconhecido: '.$e->getMessage(), 'warning');
                return false;
            }
        }
    }


    /**
     * Retorna utlimo dado de encaminhamento de uma determinada solicitacao.
     *
     * @param integer $nrorg       N?mero da Organiza??o.
     * @param integer $solicitacao N?mero da Solicita?a?.
     *
     * @return GepFluxoencasoliEntity
     */
    public function retornaUltimoDadoEncaminhamento($nrorg, $solicitacao){
        try {
            $dados = $this->_GepFluxoencasoliEntity->consultaUltimoDadoEncaminhamento($nrorg, $solicitacao);
            return $dados;
        } catch (Exception $e) {
            switch ($e->getCode()) {
            case GepFluxoencasoliEntity::SEM_REGISTRO:
                return GepFluxoencasoliEntity::SEM_REGISTRO;
            default:
                tk_message('Erro desconhecido: '.$e->getMessage(), 'warning');
                return false;
            }
        }
    }


    /**
     * Exclui os arquivos do diretorio passados como par?metro.
     *
     * @param string $lista_arquivo Diret?rio e nome do arquivo.
     *
     * @return number/boolean
     */
    public function excluiAnexosEncaminhamento($lista_arquivo){
        try {
            $count = 0;
            foreach ($lista_arquivo as $arquivo) {
                $file = new TkFile($arquivo);

                if ($file instanceof TkFile) {
                    if ($file->destroy() instanceof TkFile) {
                        $count++;
                    }
                }
            }

            return $count;
        } catch (Exception $e) {
            return false;
        }
    }


    /**   ok
     * Consulta o fluxo.
     *
     * @param integer $nrorg Numero da organizacao.
     *
     * @return string
     */
    public static function consultaFluxoSoli($nrorg){
        $sql = GepFluxoencasoliEntity::consultaFluxoSoli($nrorg);
        return $sql;
    }


    /**    ok
     * Anexa arquivos no diretorio parametrizado.
     *
     * @param string $dir_origem  Diretorio de origem.
     * @param string $dir_destino Diretorio de destino.
     *
     * @return boolean
     */
    public function incluiAnexosEncaminhamento($dir_origem, $dir_destino) {
        try {
            $dirObj = new DirectoryIterator($dir_origem);
            foreach($dirObj as $file){
                if (is_dir($file) === false) {
                    $nome_arquivo = $file->getFilename();
                    $nome_arquivo_novo = strtr(utf8_decode($nome_arquivo), utf8_decode("áàãâéêíóôõúüçÁÀÃÂÉÊÍÓÔÕÚÜÇ "), "aaaaeeiooouucAAAAEEIOOOUUC_");
                    $nome_arquivo_novo = utf8_encode($nome_arquivo_novo);

                    $FILE = new TkFile($dir_destino, false, TkFile::MODE_DIR);
                    rename($dir_origem.DIRECTORY_SEPARATOR.$nome_arquivo, $dir_destino.DIRECTORY_SEPARATOR.$nome_arquivo_novo);
                }
            }

            return true;
        } catch (Exception $e) {
            tk_warning($e->getMessage());
            return false;
        }
    }


}